EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Jobs_CreateJob',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Job_Id',
    /* Description_Text        */ 'Creates a new job.'
GO


IF OBJECT_ID('[Jobs_CreateJob]') IS NOT NULL DROP PROCEDURE [Jobs_CreateJob]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Jobs_CreateJob]
(
    @JobType_Code CHAR(1),
    @Plan_Name NVARCHAR(50),
    @Raft_Id INT,
    @Schedule_Id INT,
    @Job_Name NVARCHAR(50),   
    @JobTarget_Code CHAR(1),
    @Start_Date DATETIME,
    @Simulation_Indicator YNINDICATOR,
    @ServerIds_Csv VARCHAR(MAX),
    @ServerRoleIds_Csv VARCHAR(MAX),
    @EnvironmentIds_Csv VARCHAR(MAX),
	@TriggeredByTemplate_Id INT = NULL,
	@JobState_Code CHAR(1) = NULL,
    @Job_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
   BEGIN TRANSACTION
   SET @ServerIds_Csv = NULLIF(@ServerIds_Csv,'')
   SET @ServerRoleIds_Csv = NULLIF(@ServerRoleIds_Csv,'')
   SET @EnvironmentIds_Csv = NULLIF(@EnvironmentIds_Csv,'')
   SET @Plan_Name = NULLIF(@Plan_Name,'')
   
   IF @JobTarget_Code = 'D' AND @ServerIds_Csv IS NULL AND @JobType_Code <> 'O'
        RAISERROR('Configuration jobs targeting servers directly must specify at least one server.', 16, 1)

   IF @JobTarget_Code = 'D' AND COALESCE(@ServerRoleIds_Csv, @EnvironmentIds_Csv) IS NOT NULL
        RAISERROR('Jobs targeting servers directly cannot specify environments or roles.', 16, 1)

   IF @JobTarget_Code = 'I' AND COALESCE(@ServerRoleIds_Csv, @EnvironmentIds_Csv) IS NULL
        RAISERROR('Jobs targeting servers indirectly must specify at least one role or environment.', 16, 1)

   IF @JobTarget_Code = 'I' AND @ServerIds_Csv IS NOT NULL
        RAISERROR('Jobs targeting servers indirectly must not specify a server.', 16, 1)

   INSERT INTO [Jobs]
    (
        [JobType_Code],
        [Plan_Name],
        [Raft_Id],
        [Schedule_Id],
        [Job_Name],
        [JobTarget_Code],
        [Start_Date],
        [Simulation_Indicator],
        [JobState_Code],
		[TriggeredByTemplate_Id]
    )
    VALUES
    (   
        @JobType_Code,
        @Plan_Name,
        @Raft_Id,
        @Schedule_Id,
        @Job_Name,
        @JobTarget_Code,
        @Start_Date,
        @Simulation_Indicator,
        COALESCE(@JobState_Code, 'P'),
		@TriggeredByTemplate_Id
    )

    SET @Job_Id = SCOPE_IDENTITY()   

    IF @JobType_Code = 'C' AND @Schedule_Id IS NULL AND @Simulation_Indicator = 'N'
    BEGIN
        -- For non-simulated, unscheduled, configuration jobs, set the RemediatedWith_Job_Id for all drifted server 
        -- configurations that should be remedied by this job
        UPDATE [ServerConfigurations]
           SET [RemediatedWith_Job_Id] = @Job_Id
         WHERE [ServerConfiguration_Status_Code] = 'D'
           AND [Server_Id] IN (SELECT [Server_Id] = CAST([Value_Text] AS INT) 
                                 FROM dbo.CsvToTable(@ServerIds_Csv, ',')
                                UNION ALL
                               SELECT [Server_Id] 
                                 FROM [ServerRoleServers] SRS
                           INNER JOIN dbo.CsvToTable(@ServerRoleIds_Csv, ',') SR
                                   ON SRS.[ServerRole_Id] = CAST(SR.[Value_Text] AS INT)
                                UNION ALL
                               SELECT [Server_Id] 
                                 FROM [EnvironmentServers] ES
                           INNER JOIN dbo.CsvToTable(@EnvironmentIds_Csv, ',') E
                                   ON ES.[Environment_Id] = CAST(E.[Value_Text] AS INT))
    END
            
    INSERT INTO [JobServers] ([Job_Id], [Server_Id])
         SELECT @Job_Id, CAST([Value_Text] AS INT) 
           FROM dbo.CsvToTable(@ServerIds_Csv, ',')

    INSERT INTO [JobServerRoles] ([Job_Id], [ServerRole_Id])
         SELECT @Job_Id, CAST([Value_Text] AS INT) 
           FROM dbo.CsvToTable(@ServerRoleIds_Csv, ',')

    INSERT INTO [JobEnvironments] ([Job_Id], [Environment_Id])
         SELECT @Job_Id, CAST([Value_Text] AS INT) 
           FROM dbo.CsvToTable(@EnvironmentIds_Csv, ',')

   COMMIT
END TRY BEGIN CATCH
   IF XACT_STATE()<>0 ROLLBACK
   EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Jobs_CreateJob] TO [OtterUser_Role]
